{% extends "account/base.html" %}
{% load i18n %}

{% block content %}
<h1>
  {% trans "Setup Two-Factor Authentication" %}
</h1>

<h4>
  {% trans 'Step 1' %}:
</h4>

<p>
  {% trans 'Scan the QR code below with a token generator of your choice (for instance Google Authenticator).' %}
</p>

<img src="{{ qr_code_url }}" />

<p> Secret: {{ secret }} </p>

<h4>
  {% trans 'Step 2' %}:
</h4>

<p>
  {% trans 'Input a token generated by the app:' %}
</p>

<form method="post">
  {% csrf_token %}
  {{ form.non_field_errors }}
  {{ form.otp_token.label }}: {{ form.otp_token }}

  <button type="submit">
    {% trans 'Verify' %}
  </button>
</form>
{% endblock %}
