{% extends "account/base.html" %}
{% load i18n %}

{% block content %}
<h1>
  {% trans "Disable Two-Factor Authentication" %}
</h1>

<p>{% trans "Please enter a valid authentication token to disable two-factor authentication:" %}</p>

<form method="post">
  {% csrf_token %}
  {{ form.as_p }}
  <button type="submit">
    {% trans 'Disable Two-Factor' %}
  </button>
</form>

{% endblock %}
